@@ -1,5 +1,5 @@
 (*
- * Copyright (c) 2013-2014 David Sheets <sheets@alum.mit.edu>
+ * Copyright (c) 2013-2015 David Sheets <sheets@alum.mit.edu>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -20,8 +20,6 @@
 (** Raised when parsing of IP address syntax fails. *)
 exception Parse_error of string * string with sexp
 
-type bytes = string
-
 (** Type of ordered address scope classifications *)
 type scope =
 | Point
@@ -42,7 +40,7 @@ module V4 : sig
   (** Converts the low bytes of four int values into an abstract {! V4.t }. *)
   val make : int -> int -> int -> int -> t
 
-  (** Text string conversion *)
+  (** {3 Text string conversion} *)
 
   (** [of_string_exn ipv4_string] is the address represented
       by [ipv4_string]. Raises [Parse_error] if [ipv4_string] is not a
@@ -69,29 +67,29 @@ module V4 : sig
       the formatter [f]. *)
   val pp_hum : Format.formatter -> t -> unit
 
-  (** Bytestring conversion *)
+  (** {3 Bytestring conversion} *)
 
   (** [of_bytes_exn ipv4_octets] is the address represented
       by [ipv4_octets]. Raises [Parse_error] if [ipv4_octets] is not a
       valid representation of an IPv4 address. *)
-  val of_bytes_exn : bytes -> t
+  val of_bytes_exn : string -> t
 
   (** Same as [of_bytes_exn] but returns an option type instead of raising
       an exception. *)
-  val of_bytes : bytes -> t option
+  val of_bytes : string -> t option
 
   (** Same as [of_bytes_exn] but take an extra paramenter, the offset into
       the bytes for reading. *)
-  val of_bytes_raw : bytes -> int -> t
+  val of_bytes_raw : string -> int -> t
 
   (** [to_bytes ipv4] is a string of length 4 encoding [ipv4]. *)
-  val to_bytes : t -> bytes
+  val to_bytes : t -> string
 
   (** [to_bytes_raw ipv4 bytes offset] writes the 4 byte encoding of [ipv4]
       into [bytes] at offset [offset]. *)
-  val to_bytes_raw : t -> bytes -> int -> unit
+  val to_bytes_raw : t -> Bytes.t -> int -> unit
 
-  (** Int conversion *)
+  (** {3 Int conversion} *)
 
   (** [of_int32 ipv4_packed] is the address represented by
       [ipv4_packed]. *)
@@ -107,6 +105,21 @@ module V4 : sig
   (** [to_int16 ipv4] is the 16-bit packed encoding of [ipv4]. *)
   val to_int16 : t -> int * int
 
+  (** {3 MAC conversion} *)
+
+  (** [multicast_to_mac ipv4] is the MAC address corresponding to the
+      multicast address [ipv4]. Described by
+      {{:http://tools.ietf.org/html/rfc1112#section-6.2}RFC 1112}. *)
+  val multicast_to_mac : t -> Macaddr.t
+
+  (** {3 Host conversion} *)
+
+  (** [to_domain_name ipv4] is the domain name label list for reverse
+      lookups of [ipv4]. This includes the [.in-addr.arpa.] suffix. *)
+  val to_domain_name : t -> string list
+
+  (** {3 Common addresses} *)
+
   (** [any] is 0.0.0.0. *)
   val any : t
 
@@ -271,7 +284,7 @@ module V6 : sig
       {! V6.t }. *)
   val make : int -> int -> int -> int -> int -> int -> int -> int -> t
 
-  (** Text string conversion *)
+  (** {3 Text string conversion} *)
 
   (** [of_string_exn ipv6_string] is the address represented
       by [ipv6_string]. Raises [Parse_error] if [ipv6_string] is not a
@@ -298,29 +311,29 @@ module V6 : sig
       the formatter [f]. *)
   val pp_hum : Format.formatter -> t -> unit
 
-  (** Bytestring conversion *)
+  (** {3 Bytestring conversion} *)
 
   (** [of_bytes_exn ipv6_octets] is the address represented
       by [ipv6_octets]. Raises [Parse_error] if [ipv6_octets] is not a
       valid representation of an IPv6 address. *)
-  val of_bytes_exn : bytes -> t
+  val of_bytes_exn : string -> t
 
   (** Same as [of_bytes_exn] but returns an option type instead of raising
       an exception. *)
-  val of_bytes : bytes -> t option
+  val of_bytes : string -> t option
 
   (** Same as [of_bytes_exn] but takes an extra paramenter, the offset into
       the bytes for reading. *)
-  val of_bytes_raw : bytes -> int -> t
+  val of_bytes_raw : string -> int -> t
 
   (** [to_bytes ipv6] is a string of length 16 encoding [ipv6]. *)
-  val to_bytes : t -> bytes
+  val to_bytes : t -> string
 
   (** [to_bytes_raw ipv6 bytes offset] writes the 16 bytes encoding of [ipv6]
       into [bytes] at offset [offset]. *)
-  val to_bytes_raw : t -> bytes -> int -> unit
+  val to_bytes_raw : t -> Bytes.t -> int -> unit
 
-  (** Int conversion *)
+  (** {3 Int conversion} *)
 
   (** [of_int64 (ho, lo)] is the IPv6 address represented by two int64. *)
   val of_int64 : int64 * int64 -> t
@@ -338,6 +351,21 @@ module V6 : sig
   (** [to_int16 ipv6] is the 128-bit packed encoding of [ipv6]. *)
   val to_int16 : t -> int * int * int * int * int * int * int * int
 
+  (** {3 MAC conversion} *)
+
+  (** [multicast_to_mac ipv6] is the MAC address corresponding to the
+      multicast address [ipv6]. Described by
+      {{:https://tools.ietf.org/html/rfc2464#section-7}RFC 2464}. *)
+  val multicast_to_mac : t -> Macaddr.t
+
+  (** {3 Host conversion} *)
+
+  (** [to_domain_name ipv6] is the domain name label list for reverse
+      lookups of [ipv6]. This includes the [.ip6.arpa.] suffix. *)
+  val to_domain_name : t -> string list
+
+  (** {3 Common addresses} *)
+
   (** [unspecified] is ::. *)
   val unspecified : t
 
@@ -350,9 +378,15 @@ module V6 : sig
   (** [link_nodes] is ff02::01. *)
   val link_nodes : t
 
+  (** [interface_routers] is ff01::02. *)
+  val interface_routers : t
+
   (** [link_routers] is ff02::02. *)
   val link_routers : t
 
+  (** [site_routers] is ff05::02. *)
+  val site_routers : t
+
   (** A module for manipulating IPv6 network prefixes. *)
   module Prefix : sig
     type addr = t with sexp
@@ -432,7 +466,7 @@ module V6 : sig
     (** The Unique Local Unicast (ULA), fc00::/7. *)
     val unique_local       : t
 
-    (** Link-Local Unicast, fe80::/10. *)
+    (** Link-Local Unicast, fe80::/64. *)
     val link               : t
 
     (** The multicast network, ff00::/8. *)
@@ -445,6 +479,9 @@ module V6 : sig
         identifiers, ::/3. *)
     val noneui64_interface : t
 
+    (** Solicited-Node multicast addresses *)
+    val solicited_node : t
+
     (** [network subnet] is the address for [subnet]. *)
     val network : t -> addr
 
@@ -461,6 +498,13 @@ module V6 : sig
       hierarchy. *)
   val scope : t -> scope
 
+  (** [link_address_of_mac mac] is the link-local address for an
+      Ethernet interface derived by the IEEE MAC -> EUI-64 map with
+      the Universal/Local bit complemented for IPv6.
+      @see <https://tools.ietf.org/html/rfc2464#section-4> RFC 2464
+  *)
+  val link_address_of_mac : Macaddr.t -> t
+
   (** [is_global ipv6] is a predicate indicating whether [ipv6] globally
       addresses a node. *)
   val is_global : t -> bool
@@ -537,6 +581,15 @@ val is_multicast : t -> bool
     addresses a node. *)
 val is_private : t -> bool
 
+(** [multicast_to_mac addr] is the MAC address corresponding to the
+    multicast address [addr]. See {!V4.multicast_to_mac} and
+    {!V6.multicast_to_mac}.*)
+val multicast_to_mac : t -> Macaddr.t
+
+(** [to_domain_name addr] is the domain name label list for reverse
+    lookups of [addr]. This includes the [.arpa.] suffix. *)
+val to_domain_name : t -> string list
+
 module Prefix : sig
   type addr = t with sexp
 
